/*

*/

/**
   This simulates a first-in-first-out (stack) buffer.
*/
public class Buffer_monitor_bounded extends Buffer
{
   /**
      Create a buffer that is synchronized and bounded
      by using its internal monotor.

      @param size          the size of the buffer
      @param currentIndex  initially fill the buffer up to this index
      @throws IllegalArgumentException if currentIndex is outside of the buffer
   */
   Buffer_monitor_bounded(final int size, final int currentIndex)
   {
      super(size, currentIndex); // Initialize a partially filled buffer.
   }


   /**
      Insert the given character at the next available buffer location.

      Every time we do an operation on this buffer, it prints out its state.

      @param c  character to put in the buffer
   */
   @Override
   public synchronized void put(char c) // mutual exclusion
   throws InterruptedException
   {
      while ( this.isFull() )
      {
         this.wait();  // Use the internal monotor.
      }
      ++currentIndex;
      Thread.yield(); // Allow race conditions to happen more often.
      buf[currentIndex] = c;
      this.notify();            // Causes deadlock problem.
    //this.notifyAll();         // Solves deadlock problem.
      System.out.println(this); // Print the state of this buffer.
      System.out.flush();
   }


   /**
      Remove from the buffer the character at the buffer's current index.
      Mark the newly emptied spot in the buffer with the given character.
      Return the character that was removed from the buffer.

      Every time we do an operation on this buffer, it prints out its state.

      @param c  character to use to mark the newly emptied position in the buffer
      @return the character removed from the buffer
   */
   @Override
   public synchronized char get(char c) // mutual exclusion
   throws InterruptedException
   {
      while ( this.isEmpty() )
      {
         this.wait();  // Use the internal monotor.
      }
      char result = buf[currentIndex];
      buf[currentIndex] = c;
      Thread.yield(); // Allow race conditions to happen more often.
      --currentIndex;
      this.notify();            // Causes deadlock problem.
    //this.notifyAll();         // Solves deadlock problem.
      System.out.println(this); // Print the state of this buffer.
      System.out.flush();
      return result;
   }
}
