/*

*/

import static java.util.concurrent.TimeUnit.SECONDS;

/**
   Create an empty one element buffer and then start two
   consumer threads and one producer thread and make them
   execute in a specific order that causes one consumer
   thread to wake up the other consumer thread from the
   monitor queue, which creates a deadlock with one
   consumer and one producer stuck in the monitor queue.
*/
public class ForceDeadlock
{
   private static Buffer buffer = new Buffer_monitor_bounded(1, 0); // Empty buffer of size 1.

   public static void main(String[] args) throws InterruptedException
   {
      final Runnable producer = new Producer();
      final Runnable consumer = new Consumer();

      final Thread t1 = new Thread(producer);
      final Thread t2 = new Thread(consumer);
      final Thread t3 = new Thread(consumer);

      System.out.println(buffer); // Print the initial state of the buffer.

      t3.start();
      t2.start();
      t1.start();

      SECONDS.sleep(10); // 10 seconds

      t3.interrupt();
      t2.interrupt();
      t1.interrupt();

      t3.join(); System.out.println("t3 done");
      t2.join(); System.out.println("t2 done");
      t1.join(); System.out.println("t1 done");

      System.out.println("All threads done.");
   }


   static class Producer implements Runnable
   {
      @Override
      public void run()
      {
         try
         {
            buffer.put();
            buffer.put();
            System.out.println("Producer done.");
         }
         catch (InterruptedException e)
         {
            // Print a stack trace and then the thread dies.
            e.printStackTrace();
         }
      }
   }


   static class Consumer implements Runnable
   {
      @Override
      public void run()
      {
         try
         {
            buffer.get();
            System.out.println("Consumer done.");
         }
         catch (InterruptedException e)
         {
            // Print a stack trace and then the thread dies.
            e.printStackTrace();
         }
      }
   }
}
