/*

*/

import static java.util.concurrent.TimeUnit.SECONDS;

/**
   Create an empty one element buffer, and then create
   and start one producer and two consumer threads that
   use the buffer.

   Sometimes a consumer's signal will wake up the other
   consumer thread (instead of the producer) and this
   will cause a deadlock.

   Why does adding a second producer thread make deadlock more likely?
*/
public class ProducerConsumer_deadlock
{
   public static void main(String[] args) throws InterruptedException
   {
      final Buffer buf = new Buffer_monitor_bounded(1, 0); // Empty buffer of size 1.
      final Runnable producer = new Producer(buf, 1);
      final Runnable consumer = new Consumer(buf, 1);
      final Thread t1 = new Thread(producer);
      final Thread t2 = new Thread(consumer);
      final Thread t3 = new Thread(consumer);
      //final Thread t4 = new Thread(producer);

      System.out.println(buf); // Print the initial state of the buffer.

      t1.start();
      t2.start();
      t3.start();    // Can cause deadlock, but usually doesn't.
      //t4.start();  // Almost always causes deadlock.

      SECONDS.sleep(10);  // 10 seconds
    //SECONDS.sleep(300); // 5 minutes

      t1.interrupt();
      t2.interrupt();
      t3.interrupt();
      //t4.interrupt();

      t1.join(); System.out.println("t1 done");
      t2.join(); System.out.println("t2 done");
      t3.join(); System.out.println("t3 done");
      //t4.join(); System.out.println("t4 done");

      System.out.println("All threads done.");
   }
}
